# $Id: topic.rb,v 1.19 2003/12/10 16:42:36 not Exp $

# gsbNꗗ쐬NX
class Topics
  include DiaryModule

  @@topic = nil

  attr_accessor :showVersion
  attr_reader   :regexp
  attr_accessor :pattern
  attr_accessor :replace
  attr_reader   :diary

  def initialize
    @topicMonthFormat = '%YN%0m'
    @topicDateFormat  = '%0d(%e)'
    @regexp   = ''
    @pattern  = ''
    @replace  = ''
    @filename = ''
    @diary    = nil
    @years    = []
    @isxhtml  = nil
    @userPluginDirectory = nil
    
  end

  def skelton=(filename)
    @skelton  = filename.dup
    @filename = filename.dup
    @skelton << '.html' if /.html?$/i !~ @skelton
  end
  
  def topicMonthFormat=(str)
    @topicMonthFormat = str unless str.nil?
  end

  def topicDateFormat=(str)
    @topicDateFormat = str unless str.nil?
  end

  def regexp=(regexp)
    @regexp = regexp
  end

  def diary=(d)
    @lastestFilename = d.latestFilename
    @monthlyFilename = d.monthlyFilename
    @outputKcode     = d.outputKcode
    @monthPartNames  = d.monthPartNames
    @months          = d.months
    @allDiaries      = d.allDiaries
    @diaries         = d.diaries
    @outputDirectory = d.outputDirectory
    @logDirectory    = d.logDirectory
    @config          = d.config
    @userPluginDirectory = d.userPluginDirectory
  end

  def loadtopics
    return @@topic unless @@topic.nil?
    topic = {}
    Dir[@logDirectory + '[0-9]' * 6 + '.topic'].each{ |filename|
      open(filename){ |file|
        file.each{ |line|
          t = line.chomp.split(/\t/)
          date = t.shift
          topic[date[0..5]] = {} unless topic.key?(date[0..5])
          topic[date[0..5]][date[6..7]] = t
          for i in 0...t.size
            t[i].gsub!(/<.+?>/, '')
            t[i] = ["_t#{i+1}", t[i].chomp]
          end
        }
      }
    }
    @@topic = Marshal::dump(topic)
  end

  def filter_topics(diaries)
    loadtopics
    regexp  = Regexp::compile(@regexp)
    topic   = Marshal::load(@@topic)

    topic.delete_if{ |month, days|
      days.delete_if{ |date, topics|
        topics.delete_if{ |line| regexp !~ line[1] }
        topics.size.zero? or !diaries.include?(month + date)
      }
      days.size.zero?
    }

    years = []
    topic.keys.each { |month|
      years << month[0..3].to_s
    }
    @years = years.sort.uniq

    Marshal::dump(topic)
  end

  def enumerate_years(diaries)
    filter_topics(diaries)
    @years
  end
  
  def tohtml(diaries, putmonth = true, year = '')
    body = ''
    @diaries = diaries
    topic = Marshal::load(filter_topics(diaries))

    pattern = Regexp::compile(@pattern)

    topic.keys.sort.reverse_each{ |month|
      next unless month[0..3].index(year)
      monthlink = []
      monthhtml = ''
      
      topic[month].keys.sort.each{ |date|
        d = Time::local(month[0..3], month[4..5], date)
        monthhtml << "\t\t<li>#{d.strftime2(@topicDateFormat)} : "
        monthhtml << topic[month][date].collect!{ |t|
          monthlink << %Q!<a href="#{date2monthlyfilelink(month+date)}">#{@monthPartNames[date.to_i]}</a>!
          %Q!<a href="#{date2monthlyfilelink(month+date, true)}#{t[0]}">#{t[1].gsub(pattern, @replace)}</a>!
        }.join(' / ')
        monthhtml << "</li>\n"
      }

      if putmonth then
        m = Time::local(month[0..3], month[4..5])

        if @monthPartNames.uniq.size == 1 then
          body << %Q!\t<dt><a #{isXHTML ? 'id' : 'name'}="t#{month}" href="#{date2monthlyfilelink(month+'01')}">!
          body << "#{m.strftime2(@topicMonthFormat)}</a></dt>\n\t<dd><ul>\n"
        else
          monthlink.uniq!
          body << %Q!\t<dt><a #{isXHTML ? 'id' : 'name'}="t#{month}">#{m.strftime2(@topicMonthFormat)}</a> ( !
          body << monthlink.join(' / ')
          body << " )</dt>\n\t<dd><ul>\n"
        end
      end
      body << monthhtml
      body << "\t</ul></dd>\n" if putmonth
    }
    if putmonth then
      return "<dl>\n" + body + "</dl>\n"
    else
      return "<ul>\n" + body + "</ul>\n"
    end
  end

  def writeHTML(diaries, putmonth = true, year = nil)
    @outputKcode = 1 if (@outputKcode < 1 or @outputKcode > 4)

    if /.html?$/i !~ @filename then
      years = []
      if year.kind_of?(Array) then
        years = year
      elsif year.nil? then
        enumerate_years(diaries).each{ |y|
          years << y
        }
      else
        years = [year]
      end
      years.each{ |y|
        part_diaries = diaries.dup
        part_diaries.delete_if{ |date| date[0..3] != y }
        puthtml(@outputDirectory + @filename + y + '.html',
          @logDirectory + @filename + '.html', part_diaries, putmonth, y)
      }
    else
      puthtml(@outputDirectory + @filename, @logDirectory + @filename, diaries, putmonth)
    end

  end

  def puthtml(filename, skelton, diaries, putmonth, year = '')
    out = open(filename, "w")
    begin
      infile = open(skelton)
      infile.each { |line|
        loadplugin(line)
        if line.gsub!(/<!--body-->/i, "") then
          out.print tohtml(diaries, putmonth, year).kconv(@outputKcode)
        end
        out.print line.kconv(@outputKcode)
      }
      infile.close

      out.close
    rescue Errno::ENOENT
      $stderr.puts "\n** error: topics skelton file: '#{skelton}' not found.\n"
      return
    end
    print "make '#{filename}'\n"
  end

end

